#!/usr/bin/env perl
#
# NAME:
#	updsoa.pl - update DNS SOA record
#
# SYNOPSIS:
#	updsoa.pl [options] "soa_file" ...
#
# DESCRIPTION:
#	This script updates the serial number in "soa_file".
#	The serial number is formatted as "yyyymmddnn" where "nn"
#	differentiates between multiple updates on the same day.
#	For zones with serious flux, the '-u' uses UTC as the serial
#	number.
#	
#	The previous version of "soa_file" is left in "soa_file".old
#
#	If "soa_file" does not exist, we create it.  Various options
#	are used to control the content of the SOA record, and we also
#	look for .${Myname}rc.pl to read default values from.
#	
#	Options:
#
#	-n "ns"
#		Identifies "ns" as the master for the zone.
#
#	-l "nslist"
#		If the file "nslist" exists, it will be $INCLUDEd.
#
#	-M "email"
#		The "email" address of the owner of the zone.
#
#	-f "refresh"
#		The "refresh" interval in seconds.
#
#	-r "retry"
#		The "retry"  interval in seconds.
#
#	-e "expire"
#		The "expire" timeout (seconds).
#
#	-m "minTTL"
#		The minimum (default) ttl for the zone records.
#
#	-u	Use current time in seconds as serial number.
#
#	-T "TTL"
#		As of BIND-8.2.2 a $TTL directive is needed to set the
#		default TTL for a zone.  The minimum field of the SOA
#		record officially being used for negative caching
#		purposes.  Do not set this if using earlier versions
#		of BIND.
#
#	To cater for magic domains (eg. round-robin zones) we check
#	for each of the SOA fields in the associative arrays
#	SOA_"field".  Thus .${Myname}rc.pl might contain
#	$SOA_m{'rr.crufty.net'} = 0 to set the mimimum TTL to 0.
#
#	The command line options set the default values for the SOA fields.
#
# BUGS:
#	BIND-8.2.2 requires the $TTL record appear before any other
#	including the SOA record, otherwise it issues a warning about
#	using the SOA minimum TTL field instead.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: updsoa.pl,v 1.21 2010/11/23 05:40:33 sjg Exp $
#
#	@(#)Copyright (c) 1997-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#


if ($0 =~ m,^(.*)/([^/]+)$,) {
  $Mydir = $1;
  $Myname = $2;
} else {
  $Mydir = '.';
  $Myname = $0;
}
$Myname =~ s/\.pl//;

push(@INC, ('.', $Mydir, '/usr/local/lib/perl'));

require "$Mydir/getopts.pl";
require 'strftime.pl';
require 'dofile.pl';

$opt_n = 'ns.crufty.net';
$opt_M = 'hostmaster.crufty.net';
$opt_f = 14400;			# reFresh 4 hours
$opt_r = 1800;			# Retry 1/2 hour
$opt_e = 3600000;		# Expire 1000 hours
$opt_m = 14400;			# Minimum ttl
$opt_l = 'ns.list';
$opt_z = 'db';

&do_file(".dnsrc.pl",'.',$ENV{'HOME'},'/etc',$Mydir);
&do_file(".${Myname}rc.pl",'.',$ENV{'HOME'},'/etc',$Mydir);

&Getopts('l:n:M:f:r:e:m:z:uT:');

# make sure these do not end in '.'
$opt_n =~ s/\.$//;
$opt_M =~ s/\.$//;
# some people never read the manual
$opt_M =~ y/\@/./;
$today = &strftime('%Y%m%d', 0);

$SOA_n{'default'} = $opt_n;
$SOA_M{'default'} = $opt_M;
$SOA_f{'default'} = $opt_f;
$SOA_r{'default'} = $opt_r;
$SOA_e{'default'} = $opt_e;
$SOA_m{'default'} = $opt_m;
$SOA_u{'default'} = $opt_u;
$SOA_l{'default'} = $opt_l;
$SOA_T{'default'} = $opt_T;

&main;
exit 0;

sub main {
  foreach $soa (@ARGV) {
    if (-e $soa) {
      &upd_soa($soa);
    } else {
      &new_soa($soa);
    }
  }
}

sub setSOA {
  local(@f) = @_;
  
  local($base) = $soa;
  $base =~ s/\.soa//;
  local($domain) = $base;
  $domain =~ s,^.*/,,;

  foreach $f (@f) {
    foreach $k ($base, $domain, 'default') {
      eval "\$v = \$opt_$f = \$SOA_$f\{\$k\}";
      last if ($v ne '');
    }
  }
}

sub new_soa {
  &setSOA('n','M','f','r','e','m','u','N','T','l');
  if ("/$soa" =~ m,/\d[^/]*\.soa,) {
    $ext = ($opt_z ne '') ? $opt_z : 'rev';
   } else {
    $ext = ($opt_z ne '') ? $opt_z : 'db';
  }
  $data = $soa;
  $data =~ s/\.soa/.$ext/;
  if ($opt_u ne '') {
    $serial = time;
  } else {
    $serial = "${today}00";
  }

  $NSLIST = ($opt_l ne '' && -s $opt_l) ? "\$INCLUDE $opt_l" : "\tIN\tNS\t$opt_n.";

  if ($opt_T ne '') {
    $TTL = "\n\$TTL\t$opt_T";
  } else {
    $TTL = '';
  }
  open(OUT, "> $soa") || die;
  printf OUT ";; DO NOT EDIT THIS FILE it is maintained by magic
;; see sjg for questions...
;;$TTL
@       IN      SOA     $opt_n. $opt_M.  (
				$serial	; Last changed by - %s
				$opt_f		; Refresh
				$opt_r		; Retry
				$opt_e		; Expire 
				$opt_m )	; Minimum TTL
$NSLIST
\$INCLUDE $data\n", $ENV{'USER'};
  close(OUT);
}

sub upd_soa {
  open(IN, "< $soa") || die;
  open(OUT, "> $soa.$$") || die;

  &setSOA('u','T');

  if ($opt_T ne '') {
    $TTL = "\$TTL\t$opt_T";
  } else {
    $TTL = '';
  }
  $in_soa = 0;
  while (<IN>) {
    if (/IN\s+SOA\s/) {
      if ($TTL ne '') {
	print OUT "$TTL\n";
	$TTL = '';
      }
      $in_soa = 1;
      print OUT;
      next;
    }
    if ($in_soa) {
      $odate = '';
      if (m/^\s+(\d+)\.\d+/) {
	# BIND-9 no longer supports this!
	$odate = $1;
	$n = 0;
      } elsif (m/^\s+(\d+)(\d\d)\s/) {
	$odate = $1;
	$n = $2;
      }
      if ($odate ne '') {
	if ($opt_u ne '') {
	  printf OUT "\t\t\t\t%d\t; Last changed by - %s\n", time, $ENV{'USER'};
	} else {
	  if ($odate eq $today) {
	    $n++;
	  } else {
	    $n = 0;
	  }
	  printf OUT "\t\t\t\t$today%02d\t; Last changed by - %s\n", $n, $ENV{'USER'};
	}
	$in_soa = 0;
	next;
      }
    } elsif (m/^\$TTL\s/) {
      $TTL = '';
    }
    print OUT;
  }
  close(IN);
  close(OUT);
  rename($soa, "$soa.old");
  rename("$soa.$$", $soa);
}
